//
//  TTPeopleViewController.m
//  DAOApp
//
//  Created by Erlon Cabral on 25/02/12.
//  Copyright (c) 2012 aftertaste.cc. All rights reserved.
//

#import "TTPeopleViewController.h"

#import "TTPerson.h"

NSInteger const kGraderSegmentedControlTag = 55;

static NSString *kMenObservePath = @"menList";
static NSString *kWomenObservePath = @"womenList";
static NSString *kSearchObservePath = @"searchList";

@interface TTPeopleViewController ()
@property (nonatomic, retain) NSMutableArray *menList;
@property (nonatomic, retain) NSMutableArray *womenList;
@property (nonatomic, retain) NSMutableArray *searchList;
@property (nonatomic, assign) BOOL searching; 

- (NSInteger)graderSelectedIndex;
- (NSArray *)currentPeopleList;
- (void)peopleListWithObserveKey:(NSString *)observePath;
- (void)searchPeopleByText:(NSString *)searchText;

@end

@implementation TTPeopleViewController

@synthesize peopleSearchBar = _peopleSearchBar;

@synthesize menList = m_menList;
@synthesize womenList = m_womenList;
@synthesize searchList = _searchList;

@synthesize searching = m_searching;

#pragma mark -
#pragma mark Object creation

- (id)init
{
	self = [super initWithNibName:NSStringFromClass([TTPeopleViewController class]) bundle:nil];
	if (!self) {
		return nil;
	}
	return self;
}

#pragma mark - 
#pragma mark Memory

- (void)dealloc
{
    self.menList = nil;
    self.womenList = nil;
    self.searchList = nil;
    
    [_peopleSearchBar release];
    [super dealloc];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
    self.peopleSearchBar = nil;
    self.tableView = nil;
    
    [self removeObserver:self forKeyPath:kMenObservePath];
    [self removeObserver:self forKeyPath:kWomenObservePath];
    [self removeObserver:self forKeyPath:kSearchObservePath];
}

#pragma mark - 
#pragma mark - Instance methods (Private)

- (NSInteger)graderSelectedIndex
{
    UISegmentedControl *graderSegmentedControl = (UISegmentedControl *)[self.navigationController.toolbar viewWithTag:kGraderSegmentedControlTag];
    return graderSegmentedControl.selectedSegmentIndex;
}

- (NSArray *)currentPeopleList
{
    if (self.searching) {
        return self.searchList;
    }else if ([self graderSelectedIndex] == 0) {
        return self.menList;
    }else {
        return self.womenList;
    }
}

- (void)searchPeopleByText:(NSString *)searchText
{
    NSMutableArray *peopleList = nil;
    if ([self graderSelectedIndex] == 0) {
        peopleList = self.menList;
    }else {
        peopleList = self.womenList;
    }
    
    NSMutableArray *tempSearch = [NSMutableArray arrayWithCapacity:peopleList.count];
    
    [self willChangeValueForKey:kSearchObservePath];
    if (searchText.length > 0) {
        for (TTPerson *item in peopleList) {
            NSRange range = [[item fullName] rangeOfString:searchText options:NSCaseInsensitiveSearch];
            if (range.location != NSNotFound) {
                [tempSearch addObject:item];
            }
        }
    }else {
        tempSearch = peopleList;
    }
    
    if (self.searchList.count > 0) {
        [self.searchList removeAllObjects];
    }
    [self.searchList addObjectsFromArray:tempSearch];
    [self didChangeValueForKey:kSearchObservePath];
}

#pragma mark - 
#pragma mark - Instance methods

- (IBAction)graderSegmentedControlValueChanged:(id)sender
{
    UISegmentedControl *graderSegmentedControl = (UISegmentedControl *)sender;
    
    if (graderSegmentedControl.selectedSegmentIndex == 0) {
        [self peopleListWithObserveKey:kMenObservePath];
    }else {
        [self peopleListWithObserveKey:kWomenObservePath];
    }
    
    // filter on grader changed
    if (self.peopleSearchBar.text && ![self.peopleSearchBar.text isEqualToString:@""]) {
        [self searchPeopleByText:self.peopleSearchBar.text];
    }
}

#pragma mark -
#pragma mark KVO support

- (void)peopleListWithObserveKey:(NSString *)observeKey {
    // this will allow us as an observer to notified (see observeValueForKeyPath)
    // so we can update our UITableView
    //
    [self willChangeValueForKey:observeKey];
    
    NSInteger genderKey = ([self graderSelectedIndex] + 1);
    if ([observeKey isEqualToString:kMenObservePath]) {
        if (self.menList.count > 0) {
            [self.menList removeAllObjects];
        }
        [self.menList addObjectsFromArray:[TTPerson findAllByGenderKey:genderKey]];
    }else {
        if (self.womenList.count > 0) {
            [self.womenList removeAllObjects];
        }
        [self.womenList addObjectsFromArray:[TTPerson findAllByGenderKey:genderKey]];
    }
    
    [self didChangeValueForKey:observeKey];
}

// listen for changes to the people list.
- (void)observeValueForKeyPath:(NSString *)keyPath
                      ofObject:(id)object
                        change:(NSDictionary *)change
                       context:(void *)context {
    [self.tableView reloadData];
}

#pragma mark - 
#pragma mark View lifecycle

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.title = [NSLocalizedString(@"People", @"") stringByAppendingString:@"+"];
    
    CGRect frame = self.tableView.frame;
    frame.origin =CGPointMake(0.0, self.peopleSearchBar.frame.size.height);
    
    [self.tableView scrollRectToVisible:frame animated:NO];
    
    self.menList = [NSMutableArray array];
    self.womenList = [NSMutableArray array];
    self.searchList = [NSMutableArray array];
   
    // KVO: listen for changes to our people data source for table view updates
    [self addObserver:self forKeyPath:kMenObservePath options:0 context:NULL];
    [self addObserver:self forKeyPath:kWomenObservePath options:0 context:NULL];
    [self addObserver:self forKeyPath:kSearchObservePath options:0 context:NULL];
    
    [self peopleListWithObserveKey:kMenObservePath];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark -
#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    return [[self currentPeopleList] count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    
    if (!cell) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
        [cell setSelectionStyle:UITableViewCellSelectionStyleNone];
    }
    
    // Configure the cell...
    TTPerson *personItem = [[self currentPeopleList] objectAtIndex:indexPath.row];
    cell.textLabel.text = [personItem fullName];
    
    return cell;
}

#pragma mark - Table view delegate

//- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
//{
//}

#pragma mark - UISearchBar Delegate

- (BOOL)searchBarShouldBeginEditing:(UISearchBar *)searchBar
{
    [self.peopleSearchBar setShowsCancelButton:YES animated:YES];
    [self.navigationController setNavigationBarHidden:YES animated:YES];
    
    return YES;
}

- (void)searchBar:(UISearchBar *)searchBar textDidChange:(NSString *)searchText
{   
    self.searching = YES;
    [self searchPeopleByText:searchText];
}

- (void)searchBarSearchButtonClicked:(UISearchBar *)searchBar
{
    if (searchBar.text && ![searchBar.text isEqualToString:@""]) {
        self.searching = YES;
        //[self searchPeopleByText:searchBar.text];
    }else {
        self.searching = NO;
    }
    
    [self.navigationController setNavigationBarHidden:NO animated:YES];
    [searchBar resignFirstResponder];
}

- (void)searchBarCancelButtonClicked:(UISearchBar *) searchBar
{
    self.searching = NO;
    [self.searchList removeAllObjects];
    
    [self.peopleSearchBar setShowsCancelButton:self.searching animated:!self.searching];
    [self.navigationController setNavigationBarHidden:self.searching animated:!self.searching];
    [searchBar resignFirstResponder];
}


#pragma mark -
#pragma mark - UIScrolViewDelegate

-(void) scrollViewDidScroll:(UIScrollView *)scrollView 
{
    if (self.navigationController.navigationBarHidden) {

        

            self.peopleSearchBar.frame = CGRectMake(0,MAX(0,scrollView.contentOffset.y),320,44);


//        CGRect rect = self.peopleSearchBar.frame;
//        rect.origin.y = MIN(0, scrollView.contentOffset.y);
//        self.peopleSearchBar.frame = rect;
        
        self.tableView.frame = CGRectMake(0,self.peopleSearchBar.bounds.size.height,320,480);
    }
}

@end
